/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"

using namespace std;
using namespace FD;

class CvfdSobel;
   
DECLARE_NODE(CvfdSobel)
/*Node
 *
 * @name CvfdSobel
 * @category FDOPENCV_ORG:Morphological Operations
 * @description Calculates first, second, third or mixed image derivatives using extended Sobel operator
 *
 * @input_name IMAGEIN
 * @input_description Input image
 * @input_type CvfdImage
 *
 * @output_name IMAGEOUT
 * @output_description Image of the edges found by the function.
 * @output_type CvfdImage
 *
 * @parameter_name XORDER
 * @parameter_type int
 * @parameter_value 1
 * @parameter_description Order of the derivative x
 *
 * @parameter_name YORDER
 * @parameter_type int
 * @parameter_value 0
 * @parameter_description Order of the derivative y
 *
 * @parameter_name APERTURE_SIZE
 * @parameter_type int
 * @parameter_value 3
 * @parameter_description Size of the extended Sobel kernel, must be 1, 3, 5 or 7
 *
 END*/
   
   
class CvfdSobel : public BufferedNode {
  //Input ID
  int m_imageInID;
  //Output ID
  int m_imageOutID;
  //Parameters
  int m_xorder;
  int m_yorder;
  int m_aperture_size;
      
public:
  CvfdSobel(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    //add inputs
    m_imageInID = addInput("IMAGEIN");
    //add outputs
    m_imageOutID = addOutput("IMAGEOUT");
    //Initialize parameters
    m_xorder = dereference_cast<int>(parameters.get("XORDER"));
    m_yorder = dereference_cast<int>(parameters.get("YORDER"));
    m_aperture_size = dereference_cast<int>(parameters.get("APERTURE_SIZE"));
  }      
      
  void calculate(int output_id, int count, Buffer &out)
  {
    //Read the inputs
    RCPtr<CvfdImage> imagePtr = getInput(m_imageInID,count); 
         
    //Handle
    if(imagePtr->getImage() == NULL)
      {
	out[count] = ObjectRef(imagePtr);
	return;
      }
    CvfdImage* image = imagePtr->gray();
         
    int status = cvGetErrMode();
    cvSetErrMode( CV_ErrModeSilent );
    __BEGIN__; 
    if((image->getImage())->depth < 16)
      {
	CvfdImage* imageTemp;
	OPENCV_CALL(imageTemp = new CvfdImage(cvGetSize(image->getImage()), IPL_DEPTH_16S, 1));
	OPENCV_CALL(cvSobel(image->getImage(), imageTemp->getImage(), m_xorder, m_yorder, m_aperture_size));
	OPENCV_CALL(cvConvertScaleAbs( imageTemp->getImage(),image->getImage())); 
	delete imageTemp;
      }
    else
      {
	OPENCV_CALL(cvSobel(image->getImage(), image->getImage(), m_xorder, m_yorder, m_aperture_size));
      }   
    __END__;
    cvSetErrMode( status );         
    if( cvGetErrStatus() != CV_StsOk  )
      {
	throw new GeneralException("OPENCV - Error to sobel the image: " +  CCHAR(cvErrorStr( cvGetErrStatus() )),__FILE__,__LINE__);
      } 
    out[count] = ObjectRef(image);
  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdSobel)
};
